@@ -12,9 +12,11 @@ import re
 import logging
 from h5py import File
 import scipy.interpolate
+from scipy.interpolate import InterpolatedUnivariateSpline
 import scipy.integrate
 import numpy
 import matplotlib.pyplot
+from packaging.version import Version
 from packaging.specifiers import SpecifierSet
 
 from lisaconstants import c
@@ -156,6 +158,10 @@ class Instrument:
         self.version = meta.__version__
         self.simulated = False
 
+        # Check orbit dataset
+        if orbit_dataset not in ['tcb/ltt', 'tps/ppr']:
+            raise ValueError(f"invalid orbit dataset '{orbit_dataset}'")
+
         # Measurement sampling
         self.size = int(size)
         self.dt = float(dt)
@@ -163,13 +169,18 @@ class Instrument:
             if isinstance(orbits, str) and orbits != 'static':
                 logger.debug("Reading initial time from orbit file '%s'", orbits)
                 with File(orbits, 'r') as orbitf:
-                    if orbit_dataset == 'tcb/ltt':
-                        attr = 't0'
-                    elif orbit_dataset == 'tps/ppr':
-                        attr = 'tau0'
+                    version = Version(orbitf.attrs['version'])
+                    logger.debug("Using orbit file version %s", version)
+                    # Warn for orbit file development version
+                    if version.is_devrelease:
+                        logger.warning("You are using an orbit file in a development version")
+                    # Switch between various orbit file standards
+                    if version in SpecifierSet('== 1.*', True):
+                        self.t0 = float(orbitf.attrs['t0' if orbit_dataset == 'tcb/ltt' else 'tau0'])
+                    elif version in SpecifierSet('== 2.*', True):
+                        self.t0 = float(orbitf.attrs['t0'])
                     else:
-                        raise ValueError(f"invalid orbit dataset '{orbit_dataset}'")
-                    self.t0 = float(orbitf.attrs[attr])
+                        raise ValueError(f"unsupported orbit file version '{version}'")
             else:
                 self.t0 = 0.0
         else:
@@ -461,10 +472,16 @@ class Instrument:
             self.orbit_file = orbits
             self.orbit_dataset = orbit_dataset
             with File(self.orbit_file, 'r') as orbitf:
-                version = orbitf.attrs['version']
+                version = Version(orbitf.attrs['version'])
                 logger.debug("Using orbit file version %s", version)
-                if version in SpecifierSet('~= 1.0'):
+                # Warn for orbit file development version
+                if version.is_devrelease:
+                    logger.warning("You are using an orbit file in a development version")
+                # Switch between various orbit file standards
+                if version in SpecifierSet('== 1.*', True):
                     self.init_orbits_file_1_0(orbitf)
+                elif version in SpecifierSet('== 2.*', True):
+                    self.init_orbits_file_2_0(orbitf)
                 else:
                     raise ValueError(f"unsupported orbit file version '{version}'")
         else:
@@ -477,7 +494,7 @@ class Instrument:
             self.orbit_dataset = None
 
     def init_orbits_file_1_0(self, orbitf):
-        """Initialize orbits from an orbit file version ~= 1.0."""
+        """Initialize orbits from an orbit file version == 1.*."""
 
         def pprs(mosa):
             if self.orbit_dataset == 'tcb/ltt':
@@ -510,6 +527,33 @@ class Instrument:
             logger.error("Missing orbit information at \n%s", self.physics_t)
             raise ValueError("missing orbit information, use longer orbit file or adjust sampling") from error
 
+    def init_orbits_file_2_0(self, orbitf):
+        """Initialize orbits from an orbit file version == 2.*."""
+
+        # Prepare common interpolation method
+        times = orbitf.attrs['t0'] + numpy.arange(orbitf.attrs['size']) * orbitf.attrs['dt']
+        interpolate = lambda data, t: InterpolatedUnivariateSpline(times, data, ext='raise')(t)
+        index = {'12': 0, '23': 1, '31': 2, '13': 3, '32': 4, '21': 5}
+
+        # Interpolate necessary orbital quantities,
+        # show a helpful error message if orbit file is too short
+        try:
+            logger.debug("Interpolating proper pseudo-ranges")
+            dataset = orbitf['tcb/ltt'] if self.orbit_dataset == 'tcb/ltt' else orbitf['tps/ppr']
+            self.pprs = ForEachMOSA(lambda mosa: interpolate(dataset[:, index[mosa]], self.physics_t))
+            logger.debug("Interpolating proper pseudo-range derivatives")
+            dataset = orbitf['tcb/d_ltt'] if self.orbit_dataset == 'tcb/ltt' else orbitf['tps/d_ppr']
+            self.d_pprs = ForEachMOSA(lambda mosa: interpolate(dataset[:, index[mosa]], self.physics_t))
+            logger.debug("Interpolating proper time deviation from TCB")
+            if self.orbit_dataset == 'tcb/ltt':
+                self.tps_proper_time_deviations = ForEachSC(lambda sc: 0)
+            else:
+                dataset = orbitf['tcb/delta_tau']
+                self.tps_proper_time_deviations = ForEachSC(lambda sc: interpolate(dataset[:, sc - 1], self.physics_t))
+        except ValueError as error:
+            logger.error("Missing orbit information at \n%s", self.physics_t)
+            raise ValueError("missing orbit information, use longer orbit file or adjust sampling") from error
+
     def init_gws(self, gws):
         """Initialize gravitational-wave responses."""
         if isinstance(gws, str):
@@ -1706,14 +1750,18 @@ class Instrument:
             logger.debug("Writing downsampled test-mass beatnotes to '%s'", output)
             self.tm_carrier_offsets.write(hdf5, 'tm_carrier_offsets')
             self.tm_carrier_fluctuations.write(hdf5, 'tm_carrier_fluctuations')
+            self.tm_carriers.write(hdf5, 'tm_carriers')
             self.tm_usb_offsets.write(hdf5, 'tm_usb_offsets')
             self.tm_usb_fluctuations.write(hdf5, 'tm_usb_fluctuations')
+            self.tm_usbs.write(hdf5, 'tm_usbs')
 
             logger.debug("Writing downsampled reference beatnotes to '%s'", output)
             self.ref_carrier_offsets.write(hdf5, 'ref_carrier_offsets')
             self.ref_carrier_fluctuations.write(hdf5, 'ref_carrier_fluctuations')
+            self.ref_carriers.write(hdf5, 'ref_carriers')
             self.ref_usb_offsets.write(hdf5, 'ref_usb_offsets')
             self.ref_usb_fluctuations.write(hdf5, 'ref_usb_fluctuations')
+            self.ref_usbs.write(hdf5, 'ref_usbs')
 
         logger.info("Closing measurement file '%s'", output)
 
